from dataclasses import dataclass
from typing import Dict, List, Optional

from boltons.iterutils import chunked


@dataclass
class Config:
    """
        Config
    """

    api_key: str | None = None
    base_url: str = "https://moderator.pysols.ru"

    text_classification_model: str = "bert"
    profile_classification_model: str = "profiles_antispam_bert"
    image_classification_model: str = "nsfw_detector"


@dataclass
class BasicResponse:
    """
        Basic response class for all API responses
    """

    ok: bool
    code: int
    message: str

    def __str__(self):
        return f"""BasicResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}"
)"""


@dataclass
class TextClassResponse(BasicResponse):
    """
        Text classification response
    """

    label: int
    time_taken: float
    class_names: Dict[str, str]
    confidence: float
    unique_id: str
    balance: float
    tokens_count: int
    price: float

    def __str__(self):
        return f"""TextClassResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    label={self.label},
    time_taken={self.time_taken},
    class_names={self.class_names},
    confidence={self.confidence},
    unique_id="{self.unique_id}",
    balance={self.balance},
    tokens_count={self.tokens_count},
    price={self.price}
)"""


@dataclass
class ImageClassResponse(BasicResponse):
    """
        Image classification response
    """

    label: int
    time_taken: float
    class_names: Dict[str, str]
    confidence: float
    balance: float
    price: float

    def __str__(self):
        return f"""ImageClassResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    label={self.label},
    time_taken={self.time_taken},
    class_names={self.class_names},
    confidence={self.confidence},
    balance={self.balance},
    price={self.price}
)"""


@dataclass
class Stat:
    """
        Statistic class
    """

    date: str
    money_spent: float
    queries: int

    def __str__(self):
        return f"Stat(date=\"{self.date}\", money_spent={self.money_spent}, queries={self.queries})"


@dataclass
class StatsResponse(BasicResponse):
    """
        Stats response
    """

    stats: List[Stat]
    date_format: str

    def __str__(self):
        return f"""StatsResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    stats=[
        {",\n\t".join(str(stat) for stat in self.stats)}
    ],
    date_format="{self.date_format}"
)"""


@dataclass
class KeyInfoResponse(BasicResponse):
    """
        Key info response
    """

    balance: float

    def __str__(self):
        return f"""KeyInfoResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    balance={self.balance}
)"""

@dataclass
class PredictionResponse(BasicResponse):
    """
        Prediction response
    """

    text: str
    label: int
    confidence: float
    model: str

    def __str__(self):
        return f"""PredictionResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    text="{self.text}",
    label={self.label},
    confidence={self.confidence},
    model="{self.model}"
)"""


@dataclass
class IpsResponse(BasicResponse):
    """
        List of IPs who used the API key
    """

    ips: List[str]

    def __str__(self):
        return f"""IpsResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    ips=[
        {"\n\t".join(
            ", ".join(f"\"{ip}\"" for ip in chunk) + "," for chunk in chunked(set(self.ips), 5)
        )}
    ]
)"""


@dataclass
class Model:
    name: str
    price: float
    class_names: dict[int, str]

    def __str__(self):
        return f"""Model(
    name="{self.name}",
    price={self.price} per 128 tokens,
    class_names={self.class_names}
)"""


@dataclass
class PricesResponse(BasicResponse):
    """
        Prices response
    """

    text_classification: List[Model]
    image_classification: List[Model]
    profile_classification: List[Model]

    def __str__(self):
        return f"""PricesResponse(
    ok={self.ok},
    code={self.code},
    message="{self.message}",
    text_classification=[
        {"\n\t".join(str(model).replace("\n", "\n\t") + "," for model in self.text_classification)}
    ],
    image_classification=[
        {"\n\t".join(str(model).replace("\n", "\n\t") + "," for model in self.image_classification)}
    ],
    profile_classification=[
        {"\n\t".join(str(model).replace("\n", "\n\t") + "," for model in self.profile_classification)}
    ]
)"""

